#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

CONFIG_PATH = 'db/config.json'
SETTING_PATH = 'db/settings.json'

CONNECTED_NODE_PATH = 'db/connected_node.json'
UNL_NODES_PATH = 'db/unl_nodes.json'

TEMP_BLOCK_PATH = 'db/temp_block.decentra_network'
TEMP_ACCOUNTS_PATH = 'db/temp_accounts.decentra_network'
TEMP_ACCOUNTS_PART_PATH = 'db/temp_accounts_part.decentra_network'
TEMP_BLOCKSHASH_PATH = 'db/temp_blockshash.decentra_network'
TEMP_BLOCKSHASH_PART_PATH = 'db/temp_blockshash_part.decentra_network'

TEMP_CANDIDATE_BLOCKS_PATH = 'db/temp_candidate_blocks.decentra_network'

LOADING_BLOCK_PATH = 'db/loading_block.decentra_network'

BLOCKS_PATH = 'db/blocks/'


WALLETS_PATH = 'db/wallet_list.json'


MY_TRANSACTION_PATH = 'db/my_transaction.decentra_network'


# Export
MY_TRANSACTION_EXPORT_PATH = 'db/my_transaction.csv'

